'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var kebabCase = _interopDefault(require('kebab-case'));
var hash = _interopDefault(require('string-hash'));

function unwrapExports (x) {
	return x && x.__esModule ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var capitalizeString_1 = createCommonjsModule(function (module, exports) {
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = capitalizeString;
function capitalizeString(str) {
  return str.charAt(0).toUpperCase() + str.slice(1);
}
module.exports = exports["default"];
});

var prefixProperty_1 = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = prefixProperty;



var _capitalizeString2 = _interopRequireDefault(capitalizeString_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function prefixProperty(prefixProperties, property, style) {
  if (prefixProperties.hasOwnProperty(property)) {
    var requiredPrefixes = prefixProperties[property];
    for (var i = 0, len = requiredPrefixes.length; i < len; ++i) {
      style[requiredPrefixes[i] + (0, _capitalizeString2.default)(property)] = style[property];
    }
  }
}
module.exports = exports['default'];
});

var prefixValue_1 = createCommonjsModule(function (module, exports) {
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = prefixValue;
function prefixValue(plugins, property, value, style, metaData) {
  for (var i = 0, len = plugins.length; i < len; ++i) {
    var processedValue = plugins[i](property, value, style, metaData);

    // we can stop processing if a value is returned
    // as all plugin criteria are unique
    if (processedValue) {
      return processedValue;
    }
  }
}
module.exports = exports["default"];
});

var addNewValuesOnly_1 = createCommonjsModule(function (module, exports) {
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = addNewValuesOnly;
function addIfNew(list, value) {
  if (list.indexOf(value) === -1) {
    list.push(value);
  }
}

function addNewValuesOnly(list, values) {
  if (Array.isArray(values)) {
    for (var i = 0, len = values.length; i < len; ++i) {
      addIfNew(list, values[i]);
    }
  } else {
    addIfNew(list, values);
  }
}
module.exports = exports["default"];
});

var isObject_1 = createCommonjsModule(function (module, exports) {
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = isObject;
function isObject(value) {
  return value instanceof Object && !Array.isArray(value);
}
module.exports = exports["default"];
});

var createPrefixer_1 = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createPrefixer;



var _prefixProperty2 = _interopRequireDefault(prefixProperty_1);



var _prefixValue2 = _interopRequireDefault(prefixValue_1);



var _addNewValuesOnly2 = _interopRequireDefault(addNewValuesOnly_1);



var _isObject2 = _interopRequireDefault(isObject_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createPrefixer(_ref) {
  var prefixMap = _ref.prefixMap,
      plugins = _ref.plugins;

  function prefixAll(style) {
    for (var property in style) {
      var value = style[property];

      // handle nested objects
      if ((0, _isObject2.default)(value)) {
        style[property] = prefixAll(value);
        // handle array values
      } else if (Array.isArray(value)) {
        var combinedValue = [];

        for (var i = 0, len = value.length; i < len; ++i) {
          var processedValue = (0, _prefixValue2.default)(plugins, property, value[i], style, prefixMap);
          (0, _addNewValuesOnly2.default)(combinedValue, processedValue || value[i]);
        }

        // only modify the value if it was touched
        // by any plugin to prevent unnecessary mutations
        if (combinedValue.length > 0) {
          style[property] = combinedValue;
        }
      } else {
        var _processedValue = (0, _prefixValue2.default)(plugins, property, value, style, prefixMap);

        // only modify the value if it was touched
        // by any plugin to prevent unnecessary mutations
        if (_processedValue) {
          style[property] = _processedValue;
        }

        (0, _prefixProperty2.default)(prefixMap, property, style);
      }
    }

    return style;
  }

  return prefixAll;
}
module.exports = exports['default'];
});

var staticData = createCommonjsModule(function (module, exports) {
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = {
  plugins: [],
  prefixMap: { "appearance": ["Webkit", "Moz"], "userSelect": ["Webkit", "Moz", "ms"], "textEmphasisPosition": ["Webkit"], "textEmphasis": ["Webkit"], "textEmphasisStyle": ["Webkit"], "textEmphasisColor": ["Webkit"], "boxDecorationBreak": ["Webkit"], "clipPath": ["Webkit"], "maskImage": ["Webkit"], "maskMode": ["Webkit"], "maskRepeat": ["Webkit"], "maskPosition": ["Webkit"], "maskClip": ["Webkit"], "maskOrigin": ["Webkit"], "maskSize": ["Webkit"], "maskComposite": ["Webkit"], "mask": ["Webkit"], "maskBorderSource": ["Webkit"], "maskBorderMode": ["Webkit"], "maskBorderSlice": ["Webkit"], "maskBorderWidth": ["Webkit"], "maskBorderOutset": ["Webkit"], "maskBorderRepeat": ["Webkit"], "maskBorder": ["Webkit"], "maskType": ["Webkit"], "textDecorationStyle": ["Webkit"], "textDecorationSkip": ["Webkit"], "textDecorationLine": ["Webkit"], "textDecorationColor": ["Webkit"], "filter": ["Webkit"], "fontFeatureSettings": ["Webkit"], "breakAfter": ["Webkit", "Moz", "ms"], "breakBefore": ["Webkit", "Moz", "ms"], "breakInside": ["Webkit", "Moz", "ms"], "columnCount": ["Webkit", "Moz"], "columnFill": ["Webkit", "Moz"], "columnGap": ["Webkit", "Moz"], "columnRule": ["Webkit", "Moz"], "columnRuleColor": ["Webkit", "Moz"], "columnRuleStyle": ["Webkit", "Moz"], "columnRuleWidth": ["Webkit", "Moz"], "columns": ["Webkit", "Moz"], "columnSpan": ["Webkit", "Moz"], "columnWidth": ["Webkit", "Moz"], "flex": ["Webkit"], "flexBasis": ["Webkit"], "flexDirection": ["Webkit"], "flexGrow": ["Webkit"], "flexFlow": ["Webkit"], "flexShrink": ["Webkit"], "flexWrap": ["Webkit"], "alignContent": ["Webkit"], "alignItems": ["Webkit"], "alignSelf": ["Webkit"], "justifyContent": ["Webkit"], "order": ["Webkit"], "transform": ["Webkit"], "transformOrigin": ["Webkit"], "transformOriginX": ["Webkit"], "transformOriginY": ["Webkit"], "backfaceVisibility": ["Webkit"], "perspective": ["Webkit"], "perspectiveOrigin": ["Webkit"], "transformStyle": ["Webkit"], "transformOriginZ": ["Webkit"], "animation": ["Webkit"], "animationDelay": ["Webkit"], "animationDirection": ["Webkit"], "animationFillMode": ["Webkit"], "animationDuration": ["Webkit"], "animationIterationCount": ["Webkit"], "animationName": ["Webkit"], "animationPlayState": ["Webkit"], "animationTimingFunction": ["Webkit"], "backdropFilter": ["Webkit"], "fontKerning": ["Webkit"], "scrollSnapType": ["Webkit", "ms"], "scrollSnapPointsX": ["Webkit", "ms"], "scrollSnapPointsY": ["Webkit", "ms"], "scrollSnapDestination": ["Webkit", "ms"], "scrollSnapCoordinate": ["Webkit", "ms"], "shapeImageThreshold": ["Webkit"], "shapeImageMargin": ["Webkit"], "shapeImageOutside": ["Webkit"], "hyphens": ["Webkit", "Moz", "ms"], "flowInto": ["Webkit", "ms"], "flowFrom": ["Webkit", "ms"], "regionFragment": ["Webkit", "ms"], "textAlignLast": ["Moz"], "tabSize": ["Moz"], "wrapFlow": ["ms"], "wrapThrough": ["ms"], "wrapMargin": ["ms"], "gridTemplateColumns": ["ms"], "gridTemplateRows": ["ms"], "gridTemplateAreas": ["ms"], "gridTemplate": ["ms"], "gridAutoColumns": ["ms"], "gridAutoRows": ["ms"], "gridAutoFlow": ["ms"], "grid": ["ms"], "gridRowStart": ["ms"], "gridColumnStart": ["ms"], "gridRowEnd": ["ms"], "gridRow": ["ms"], "gridColumn": ["ms"], "gridColumnEnd": ["ms"], "gridColumnGap": ["ms"], "gridRowGap": ["ms"], "gridArea": ["ms"], "gridGap": ["ms"], "textSizeAdjust": ["Webkit", "ms"], "transitionDelay": ["Webkit"], "transitionDuration": ["Webkit"], "transitionProperty": ["Webkit"], "transitionTimingFunction": ["Webkit"] }
};
module.exports = exports["default"];
});

var cursor_1 = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = cursor;
var prefixes = ['-webkit-', '-moz-', ''];

var values = {
  'zoom-in': true,
  'zoom-out': true,
  grab: true,
  grabbing: true
};

function cursor(property, value) {
  if (property === 'cursor' && values.hasOwnProperty(value)) {
    return prefixes.map(function (prefix) {
      return prefix + value;
    });
  }
}
module.exports = exports['default'];
});

var isPrefixedValue_1 = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = isPrefixedValue;

var regex = /-webkit-|-moz-|-ms-/;

function isPrefixedValue(value) {
  return typeof value === 'string' && regex.test(value);
}
module.exports = exports['default'];
});

var crossFade_1 = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = crossFade;



var _isPrefixedValue2 = _interopRequireDefault(isPrefixedValue_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// http://caniuse.com/#search=cross-fade
var prefixes = ['-webkit-', ''];
function crossFade(property, value) {
  if (typeof value === 'string' && !(0, _isPrefixedValue2.default)(value) && value.indexOf('cross-fade(') > -1) {
    return prefixes.map(function (prefix) {
      return value.replace(/cross-fade\(/g, prefix + 'cross-fade(');
    });
  }
}
module.exports = exports['default'];
});

var filter_1 = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = filter;



var _isPrefixedValue2 = _interopRequireDefault(isPrefixedValue_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// http://caniuse.com/#feat=css-filter-function
var prefixes = ['-webkit-', ''];
function filter(property, value) {
  if (typeof value === 'string' && !(0, _isPrefixedValue2.default)(value) && value.indexOf('filter(') > -1) {
    return prefixes.map(function (prefix) {
      return value.replace(/filter\(/g, prefix + 'filter(');
    });
  }
}
module.exports = exports['default'];
});

var flex_1 = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = flex;
var values = {
  flex: true,
  'inline-flex': true
};

function flex(property, value) {
  if (property === 'display' && values.hasOwnProperty(value)) {
    return ['-webkit-box', '-moz-box', '-ms-' + value + 'box', '-webkit-' + value, value];
  }
}
module.exports = exports['default'];
});

var flexboxOld_1 = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = flexboxOld;
var alternativeValues = {
  'space-around': 'justify',
  'space-between': 'justify',
  'flex-start': 'start',
  'flex-end': 'end',
  'wrap-reverse': 'multiple',
  wrap: 'multiple'
};

var alternativeProps = {
  alignItems: 'WebkitBoxAlign',
  justifyContent: 'WebkitBoxPack',
  flexWrap: 'WebkitBoxLines'
};

function flexboxOld(property, value, style) {
  if (property === 'flexDirection' && typeof value === 'string') {
    if (value.indexOf('column') > -1) {
      style.WebkitBoxOrient = 'vertical';
    } else {
      style.WebkitBoxOrient = 'horizontal';
    }
    if (value.indexOf('reverse') > -1) {
      style.WebkitBoxDirection = 'reverse';
    } else {
      style.WebkitBoxDirection = 'normal';
    }
  }
  if (alternativeProps.hasOwnProperty(property)) {
    style[alternativeProps[property]] = alternativeValues[value] || value;
  }
}
module.exports = exports['default'];
});

var gradient_1 = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = gradient;



var _isPrefixedValue2 = _interopRequireDefault(isPrefixedValue_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var prefixes = ['-webkit-', '-moz-', ''];

var values = /linear-gradient|radial-gradient|repeating-linear-gradient|repeating-radial-gradient/;

function gradient(property, value) {
  if (typeof value === 'string' && !(0, _isPrefixedValue2.default)(value) && values.test(value)) {
    return prefixes.map(function (prefix) {
      return prefix + value;
    });
  }
}
module.exports = exports['default'];
});

var imageSet_1 = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = imageSet;



var _isPrefixedValue2 = _interopRequireDefault(isPrefixedValue_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// http://caniuse.com/#feat=css-image-set
var prefixes = ['-webkit-', ''];
function imageSet(property, value) {
  if (typeof value === 'string' && !(0, _isPrefixedValue2.default)(value) && value.indexOf('image-set(') > -1) {
    return prefixes.map(function (prefix) {
      return value.replace(/image-set\(/g, prefix + 'image-set(');
    });
  }
}
module.exports = exports['default'];
});

var position_1 = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = position;
function position(property, value) {
  if (property === 'position' && value === 'sticky') {
    return ['-webkit-sticky', 'sticky'];
  }
}
module.exports = exports['default'];
});

var sizing_1 = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = sizing;
var prefixes = ['-webkit-', '-moz-', ''];

var properties = {
  maxHeight: true,
  maxWidth: true,
  width: true,
  height: true,
  columnWidth: true,
  minWidth: true,
  minHeight: true
};
var values = {
  'min-content': true,
  'max-content': true,
  'fill-available': true,
  'fit-content': true,
  'contain-floats': true
};

function sizing(property, value) {
  if (properties.hasOwnProperty(property) && values.hasOwnProperty(value)) {
    return prefixes.map(function (prefix) {
      return prefix + value;
    });
  }
}
module.exports = exports['default'];
});

var uppercasePattern = /[A-Z]/g;
var msPattern = /^ms-/;
var cache = {};

function hyphenateStyleName(string) {
    return string in cache
    ? cache[string]
    : cache[string] = string
      .replace(uppercasePattern, '-$&')
      .toLowerCase()
      .replace(msPattern, '-ms-');
}

var index$1 = hyphenateStyleName;

var hyphenateProperty_1 = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = hyphenateProperty;



var _hyphenateStyleName2 = _interopRequireDefault(index$1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function hyphenateProperty(property) {
  return (0, _hyphenateStyleName2.default)(property);
}
module.exports = exports['default'];
});

var transition_1 = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = transition;



var _hyphenateProperty2 = _interopRequireDefault(hyphenateProperty_1);



var _isPrefixedValue2 = _interopRequireDefault(isPrefixedValue_1);



var _capitalizeString2 = _interopRequireDefault(capitalizeString_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var properties = {
  transition: true,
  transitionProperty: true,
  WebkitTransition: true,
  WebkitTransitionProperty: true,
  MozTransition: true,
  MozTransitionProperty: true
};


var prefixMapping = {
  Webkit: '-webkit-',
  Moz: '-moz-',
  ms: '-ms-'
};

function prefixValue(value, propertyPrefixMap) {
  if ((0, _isPrefixedValue2.default)(value)) {
    return value;
  }

  // only split multi values, not cubic beziers
  var multipleValues = value.split(/,(?![^()]*(?:\([^()]*\))?\))/g);

  for (var i = 0, len = multipleValues.length; i < len; ++i) {
    var singleValue = multipleValues[i];
    var values = [singleValue];
    for (var property in propertyPrefixMap) {
      var dashCaseProperty = (0, _hyphenateProperty2.default)(property);

      if (singleValue.indexOf(dashCaseProperty) > -1 && dashCaseProperty !== 'order') {
        var prefixes = propertyPrefixMap[property];
        for (var j = 0, pLen = prefixes.length; j < pLen; ++j) {
          // join all prefixes and create a new value
          values.unshift(singleValue.replace(dashCaseProperty, prefixMapping[prefixes[j]] + dashCaseProperty));
        }
      }
    }

    multipleValues[i] = values.join(',');
  }

  return multipleValues.join(',');
}

function transition(property, value, style, propertyPrefixMap) {
  // also check for already prefixed transitions
  if (typeof value === 'string' && properties.hasOwnProperty(property)) {
    var outputValue = prefixValue(value, propertyPrefixMap);
    // if the property is already prefixed
    var webkitOutput = outputValue.split(/,(?![^()]*(?:\([^()]*\))?\))/g).filter(function (val) {
      return !/-moz-|-ms-/.test(val);
    }).join(',');

    if (property.indexOf('Webkit') > -1) {
      return webkitOutput;
    }

    var mozOutput = outputValue.split(/,(?![^()]*(?:\([^()]*\))?\))/g).filter(function (val) {
      return !/-webkit-|-ms-/.test(val);
    }).join(',');

    if (property.indexOf('Moz') > -1) {
      return mozOutput;
    }

    style['Webkit' + (0, _capitalizeString2.default)(property)] = webkitOutput;
    style['Moz' + (0, _capitalizeString2.default)(property)] = mozOutput;
    return outputValue;
  }
}
module.exports = exports['default'];
});

var index = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});



var _createPrefixer2 = _interopRequireDefault(createPrefixer_1);



var _staticData2 = _interopRequireDefault(staticData);



var _cursor2 = _interopRequireDefault(cursor_1);



var _crossFade2 = _interopRequireDefault(crossFade_1);



var _filter2 = _interopRequireDefault(filter_1);



var _flex2 = _interopRequireDefault(flex_1);



var _flexboxOld2 = _interopRequireDefault(flexboxOld_1);



var _gradient2 = _interopRequireDefault(gradient_1);



var _imageSet2 = _interopRequireDefault(imageSet_1);



var _position2 = _interopRequireDefault(position_1);



var _sizing2 = _interopRequireDefault(sizing_1);



var _transition2 = _interopRequireDefault(transition_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var plugins = [_crossFade2.default, _cursor2.default, _filter2.default, _flexboxOld2.default, _gradient2.default, _imageSet2.default, _position2.default, _sizing2.default, _transition2.default, _flex2.default];

exports.default = (0, _createPrefixer2.default)({
  prefixMap: _staticData2.default.prefixMap,
  plugins: plugins
});
module.exports = exports['default'];
});

var prefix = unwrapExports(index);

/**
 * Check if a string begins with the characters of a specified string.
 * @param {string} str - Original string.
 * @param {string} searchString - Characters to be searched.
 * @returns {boolean} true if str starts with searchString
 * @example
 * // returns true
 * startsWith('Hello World', 'Hell')
 * @example
 * // returns false
 * startsWith('Hello World', 'Heaven')
 */
var startsWith = function (str, searchString) { return str.indexOf(searchString) === 0; };

/**
 * Check if the given string is a pseudo selector/class
 * @param {string} str - String to be tested.
 * @return {boolean} true if str is a pseudo selector/class
 * @example
 * // returns true
 * isPseudoSelector(':after')
 * @example
 * // returns false
 * isPseudoSelector('borderRadius')
 */
var isPseudoSelector = function (str) { return startsWith(str, ':'); };

/**
 * Check if the given string is a media querie.
 * @param {string} str - String to be tested.
 * @return {boolean} true if str is a media querie
 * @example
 * // returns true
 * isMediaQuerie('@media (min-width: 700px)')
 * // returns false
 * isMediaQuerie('borderRadius')
 */
var isMediaQuery = function (str) { return startsWith(str, '@media'); };

/**
 * Check if the given argument is an array.
 * @param {*} arr - argument to be checked if it is an Array.
 * @returns {boolean} true if the arr is an Array.
 * @example
 * // returns true
 * isArray([])
 * @example
 * // returns false
 * isArray({})
 */
var isArray = function (arr) { return Object.prototype.toString.call(arr) === '[object Array]'; };

/**
 * Build a single css declaration.
 * @param {string} property - The property of the css declaration.
 * @param {string} value - The value of the css declaration.
 * @returns {string} formated css declaration.
 * @example
 * // returns 'width:100%;'
 * buildSingleDeclaration('width', '100%')
 */
var buildSingleDeclaration = function (property, value) { return ((kebabCase(property)) + ":" + value + ";"); };

/**
 * Build the css declarations.
 * @param {Object} declarationsObj - A style object.
 * @returns {string} formated css declarations.
 * @example
 * // returns 'border-radius-2px;'
 * buildDeclarations({borderRadius: '2px'})
 * @example
 * // returns '-webkit-user-select:none;'
 * buildDeclarations({WebkitUserSelect: 'none'})
 * @example
 * // returns 'display:-webkit-flex;display:flex;'
 * buildDeclarations({display: ['-webkit-flex', 'flex']})
 */
var buildDeclarations = function (declarationsObj) { return Object.keys(declarationsObj)
    .filter(function (declaration) { return !isPseudoSelector(declaration); })
    .filter(function (declaration) { return !isMediaQuery(declaration); })
    .reduce(function (previousValue, currentValue) {
      if (isArray(declarationsObj[currentValue])) {
        return (
          previousValue +
          declarationsObj[currentValue].reduce(
            function (a, b) { return a + buildSingleDeclaration(currentValue, b); },
            ''
          )
        )
      }
      return (
        previousValue +
        buildSingleDeclaration(currentValue, declarationsObj[currentValue])
      )
    }, ''); };

/**
 * Build a css rule.
 * @param {string} selector - A class name.
 * @param {Object} declarations - CSS declarations.
 * @returns {string} formated css rule.
 * @example
 * // returns '.button{border-radius-2px;}'
 * buildDeclarations('button', {borderRadius:'2px'})
 */
var buildRule = function (selector, declarations, globalSelector) {
  if ( globalSelector === void 0 ) globalSelector = false;

  return ("" + (globalSelector ? '' : '.') + selector + "{" + (buildDeclarations(prefix(declarations))) + "}")
};

/**
 * Build a css rule for pseudo selectors.
 * @param {string} selector - A class name.
 * @param {Object} declarations - CSS declarations.
 * @returns {string} formated css rule.
 * @example
 * // returns '.button:before{border-radius-2px;}'
 * buildDeclarations('button', {':before': {borderRadius:'2px'})
 */
var buildPseudoRule = function (
  selector,
  declarations,
  globalSelector
) {
  if ( globalSelector === void 0 ) globalSelector = false;

  return Object.keys(declarations)
    .filter(isPseudoSelector)
    .map(function (declaration) { return buildRule(
        ("" + selector + declaration),
        declarations[declaration],
        globalSelector
      ); }
    )
    .reduce(function (previous, current) { return previous + current; }, '')
};

/**
 * Build a css rule for a media query.
 * @param {string} selector - A class name.
 * @param {Object} declarations - CSS declarations.
 * @returns {string} formated css rule.
 * @example
 * // returns @media (min-width: 700px){app{color:blue;}}
 * buildMediaQueryRule('app', {'@media (min-width: 700px)': {color: 'blue'}})
 */
var buildMediaQueryRule = function (
  selector,
  declarations,
  globalSelector
) {
  if ( globalSelector === void 0 ) globalSelector = false;

  return Object.keys(declarations)
    .filter(isMediaQuery)
    .map(
      function (declaration) { return (declaration + "{" + (buildRule(selector, declarations[declaration], globalSelector)) + "}"); }
    )
    .reduce(function (previous, current) { return previous + current; }, '')
};

/**
 * Build a css rule.
 * @param {string} selector - A class name.
 * @param {Object} declarations - CSS declarations.
 * @returns {string} formated css rule.
 * @example
 * // returns '.button{border-radius: 2px;}.button:after{content:""}'
 * buildDeclarations('button', {
 *   borderRadius: '2px',
 *   ':after': {
 *     content: '""'
 *   }
 * })
 */
var buildStylesheet = function (selector, declarations, globalSelector) {
  if ( globalSelector === void 0 ) globalSelector = false;

  var pseudoSelectors = buildPseudoRule(
    selector,
    declarations,
    globalSelector
  );
  var normalSelector = buildRule(selector, declarations, globalSelector);
  var mediaQuery = buildMediaQueryRule(selector, declarations, globalSelector);

  return normalSelector + pseudoSelectors + mediaQuery
};

/**
 * Build a css selector with hashes.
 * @param {string} className - A class name.
 * @param {Object} declarationsObj - A object of css declarations.
 * @returns {string} a class name with hashes.
 * @example
 * // returns 'app_1nlygtv'
 * buildSelector('app', {borderRadius: '2px', width: '100%'})
 */
var buildSelector = function (className, declarationsObj) {
  return className + '_' + hash(JSON.stringify(declarationsObj)).toString(36)
};

/**
 * Build the steps of a keyframe.
 * @param {Object} steps - CSS declarations.
 * @returns {string} valid css keuframe steps.
 * @example
 * // returns 'from{transform:rotate(0deg);}to{transform:rotate(3600deg);}'
 * buildKeyframeSteps({
 *   from: {
 *     transform: 'rotate(0deg)'
 *   },
 *   to: {
 *     transform: 'rotate(360deg)'
 *   }
 * })
 */
var buildKeyframeSteps = function (steps) { return Object.keys(steps)
    .map(function (step) { return (step + "{" + (buildDeclarations(steps[step])) + "}"); })
    .join(''); };

/**
 * Build a keyframes rule.
 * @param {string} animationName - the name of the animation to be created.
 * @param {Object} steps - Steps of the keyframes animation.
 * @returns {string} formated keyframes rule.
 * @example
 * // returns '@keyframes rotate{from{opacity:0;}to{opacity:1;}}'
 * buildKeyframe('rotate', {
 *   from: {
 *     opacity: 0
 *   },
 *   to: {
 *     opacity: 1
 *   }
 * })
 */
var buildKeyframe = function (animationName, steps) {
  return ("@keyframes " + animationName + "{" + (buildKeyframeSteps(steps)) + "}")
};

/**
 * Build a font-face rule.
 * @param {Object} rules - CSS declarations.
 * @returns {string} formated font-face rule.
 * @example
 * // returns '@font-face{font-family:MyHelvetica;src:url(MgOpenModerna.ttf)}'
 * buildFontFace({
 *   fontFamily: 'MyHelvetica',
 *   src: 'url(MgOpenModernaBold.ttf)'
 * })
 */
var buildFontFace = function (rules) { return ("@font-face{" + (buildDeclarations(rules)) + "}"); };

var style = document.createElement('style');
style.dataset.stylord = '';
document.head.appendChild(style);

/**
 * Create a stylesheet and inject it to the head of the application.
 * @param {Object} rules - CSS rules to be rendered.
 * @returns {Object} the class names of the css modules created.
 * @example
 * // returns {modal: 'modal_hgdfgf', button: 'button_guyhg'}
 * createStyles({
 *   modal: {
 *     width: '100%'
 *   },
 *   button: {
 *     borderRadius: '2px'
 *   }
 * })
 */
var createStyles = function (rules) {
  var styles = Object.keys(rules)
    .map(function (rule) { return ({
      componentName: rule,
      rule: rules[rule],
      className: buildSelector(rule, rules[rule])
    }); })
    .map(function (ref) {
      var className = ref.className;
      var rule = ref.rule;
      var componentName = ref.componentName;

      return ({
      componentName: componentName,
      className: className,
      rule: buildStylesheet(className, rule)
    });
  });

  var modules = styles.reduce(
    function (previous, current) { return (( obj = Object.assign({}, previous), obj[current.componentName] = current.className, obj ))
      var obj; },
    {}
  );

  var stylesheet = styles.reduce(function (previous, current) {
    return previous + current.rule
  }, '');

  style.appendChild(document.createTextNode(stylesheet));

  return modules
};

/**
 * Create a keyframe animation and inject it to the head of the application.
 * @param {Object} rules - CSS keyframe to be create.
 * @returns {Object} the names of the keyframes created.
 * @example
 * // returns {fade: 'fade_hgdfgf'}
 * stylord({
 *   fade: {
 *     from: {
 *       opacity: 1
 *     },
 *     to: {
 *       opacity: 0
 *     }
 *   }
 * })
 */
var createKeyframes = function (rules) {
  var keyframes = Object.keys(rules)
    .map(function (rule) { return ({
      steps: rules[rule],
      keyframe: rule,
      animationName: buildSelector(rule, rules[rule])
    }); })
    .map(function (ref) {
      var keyframe = ref.keyframe;
      var steps = ref.steps;
      var animationName = ref.animationName;

      return ({
      animationName: animationName,
      keyframe: keyframe,
      rule: buildKeyframe(animationName, steps)
    });
  });

  var modules = keyframes.reduce(
    function (previous, current) { return (( obj = Object.assign({}, previous), obj[current.keyframe] = current.animationName, obj ))
      var obj; },
    {}
  );

  var stylesheet = keyframes.reduce(function (previous, current) {
    return previous + current.rule
  }, '');

  style.appendChild(document.createTextNode(stylesheet));

  return modules
};

/**
 * Create a font-face and inject it to the head of the application.
 * @param {Object} rules - CSS font-face rules to be rendered.
 * @example
 * createFontFace({
 *   fontFamily: 'MyHelvetica',
 *   src: 'local("Helvetica Neue Bold"), url(MgOpenModernaBold.ttf)',
 *   fontWeight: 'bold'
 * })
 */
var createFontFace = function (rules) {
  style.appendChild(document.createTextNode(buildFontFace(rules)));
};

/**
 * Create a global css and inject it to the head of the application.
 * @param {Object} rules - CSS global rules to be rendered.
 * @example
 * createGlobals({
 *   '*': {
 *     border: 0,
 *     boxSizing: 'inherit',
 *     margin: 0,
 *     padding: 0,
 *     outline: 0,
 *     verticalAlign: 'baseline'
 *   },
 *   body: {
 *     boxSizing: 'border-box',
 *     lineHeight: 1.5
 *   }
 * })
 */
var createGlobals = function (rules) {
  var globalStyles = Object.keys(rules)
    .map(function (rule) { return buildStylesheet(rule, rules[rule], true); })
    .join('');

  style.appendChild(document.createTextNode(globalStyles));
};

exports.createStyles = createStyles;
exports.createKeyframes = createKeyframes;
exports.createFontFace = createFontFace;
exports.createGlobals = createGlobals;
